==> chainwallet-1.0.2.tar.gz: lib/src/core/types/signed/SignedTransaction.dart <==
import 'dart:core';
import "dart:typed_data";

import "../../coretypes/STObject.dart";
import "../../serialized/enums/TransactionType.dart";

import 'package:chainwallet/src/core/coretypes/STObject.dart';
import 'package:chainwallet/src/core/types/tx/Transaction.dart';

import 'package:chainwallet/src/core/coretypes/hash/Hash256.dart';

import 'package:chainwallet/src/ecdsa/IkeyPair.dart';
import 'package:chainwallet/src/ecdsa/Seed.dart';

import 'package:chainwallet/src/core/coretypes/Amount.dart';

import 'package:chainwallet/src/core/coretypes/uint/UInt32.dart';
import 'package:chainwallet/src/core/coretypes/Blob.dart';

import 'package:chainwallet/src/core/coretypes/hash/prefixes/HashPrefix.dart';

import 'package:chainwallet/src/utils/Util.dart';
import 'package:chainwallet/src/core/serialized/ByteList.dart';
import 'package:chainwallet/src/core/serialized/BytesSink.dart';
import 'package:chainwallet/src/core/serialized/MultiSink.dart';
import 'package:pointycastle/digests/sha512.dart';



import 'package:collection/collection.dart';



class SignedTransaction {

  Transaction txn;
  Hash256 hash;

  Uint8List signingData;
  Uint8List previousSigningData;
  String    tx_blob;

  SignedTransaction(Transaction of) {

//      var tmp = of.toBytes();
//      print("SignedTransaction\r\n");
//      print( Util.formatBytesAsHexString(tmp));

      STObject  tx = STObject.translate.fromBytes(of.toBytes());
      txn =  tx as Transaction;
  }


  void multiSign(String base58Secret) =>  null;

  void multiSignFrom(IKeyPair keyPair) => null;

//  public void multiSignPrepare(IKeyPair keyPair,
//      Amount fee,
//      UInt32 Sequence,
//      UInt32 lastLedgerSequence)

    void sign(String base58Secret) {
      signFromKeyPair(Seed.fromBase58(base58Secret).keyPair());
    }

    static SignedTransaction fromTx(Transaction tx) {

      var signed = new SignedTransaction(tx);
      return signed;
    }


  void signFromKeyPair(IKeyPair keyPair) {
     prepare(keyPair, null, null, null);
  }

  void prepare(IKeyPair keyPair,
      Amount fee,
      UInt32 Sequence,
      UInt32 lastLedgerSequence) {

    Blob pubKey = new Blob(keyPair.canonicalPubBytes());

    // This won't always be specified
//    if (lastLedgerSequence != null) {
//      txn.put(UInt32.LastLedgerSequence, lastLedgerSequence);
//    }
//    if (Sequence != null) {
//      txn.put(UInt32.Sequence, Sequence);
//    }
//    if (fee != null) {
//      txn.put(Amount.Fee, fee);
//    }

    txn.signingPubKeyFromBlob(pubKey);

    if (Transaction.CANONICAL_FLAG_DEPLOYED) {
        txn.setCanonicalSignatureFlag();
    }

    txn.checkFormat();
    signingData = txn.signingData();

    var hexStr =  Util.formatBytesAsHexString(signingData);
    //print("签名前数据    : $hexStr");

    Function eq = const ListEquality().equals;
    if (previousSigningData != null && eq(signingData, previousSigningData)) {
      return;
    }
    try {

      var  tmpSign =   keyPair.signMessage(signingData);

      txn.txnSignature(new Blob(keyPair.signMessage(signingData)));

      BytesList blob = new BytesList();

    //  HalfSha512 id = HalfSha512.prefixed256(HashPrefix.transactionID);

  //    SHA512halfDigest digest = new SHA512halfDigest();

      var sha512 = new SHA512Digest();
      sha512.update(HashPrefix.transactionID.bytes(), 0, HashPrefix.transactionID.bytes().length);


      List<BytesSink> listBytesSink = new List<BytesSink>();

      listBytesSink.add(blob);

      txn.toBytesSink(new MultiSink(listBytesSink));

      tx_blob = blob.bytesHex();
      
     // print("tx_blob : $tx_blob");


      hash = Hash256.prefixedHalfSha512(HashPrefix.transactionID, blob.bytes());


//      var hexHash1 = hash.toString();
//      var hexHash = Util.formatBytesAsHexString(hash.toBytes());

     // print("hash : $hexHash1");

    } on Exception catch (e){

         previousSigningData = null;
         throw new Exception(e);
    }

      previousSigningData = signingData;

  }

  TransactionType transactionType() {
    return txn.transactionType();
  }

}

==> chainwallet-1.0.4.tar.gz: lib/src/core/types/signed/SignedTransaction.dart <==
import 'dart:core';
import "dart:typed_data";

import "../../coretypes/STObject.dart";
import "../../serialized/enums/TransactionType.dart";

import 'package:chainwallet/src/core/coretypes/STObject.dart';
import 'package:chainwallet/src/core/types/tx/Transaction.dart';

import 'package:chainwallet/src/core/coretypes/hash/Hash256.dart';

import 'package:chainwallet/src/ecdsa/IkeyPair.dart';
import 'package:chainwallet/src/ecdsa/Seed.dart';

import 'package:chainwallet/src/core/coretypes/Amount.dart';

import 'package:chainwallet/src/core/coretypes/uint/UInt32.dart';
import 'package:chainwallet/src/core/coretypes/Blob.dart';

import 'package:chainwallet/src/core/coretypes/hash/prefixes/HashPrefix.dart';

import 'package:chainwallet/src/utils/Util.dart';
import 'package:chainwallet/src/core/serialized/ByteList.dart';
import 'package:chainwallet/src/core/serialized/BytesSink.dart';
import 'package:chainwallet/src/core/serialized/MultiSink.dart';
import 'package:pointycastle/digests/sha512.dart';

import 'package:collection/collection.dart';

class SignedTransaction {
  Transaction txn;
  Hash256 hash;

  Uint8List signingData;
  Uint8List previousSigningData;
  String tx_blob;

  SignedTransaction(Transaction of) {
//      var tmp = of.toBytes();
//      print("SignedTransaction\r\n");
//      print( Util.formatBytesAsHexString(tmp));

    STObject tx = STObject.translate.fromBytes(of.toBytes());
    txn = tx as Transaction;
  }

  void multiSign(String base58Secret) => null;

  void multiSignFrom(IKeyPair keyPair) => null;

//  public void multiSignPrepare(IKeyPair keyPair,
//      Amount fee,
//      UInt32 Sequence,
//      UInt32 lastLedgerSequence)

  void sign(String base58Secret) {
    signFromKeyPair(Seed.fromBase58(base58Secret).keyPair());
  }

  static SignedTransaction fromTx(Transaction tx) {
    var signed = new SignedTransaction(tx);
    return signed;
  }

  void signFromKeyPair(IKeyPair keyPair) {
    prepare(keyPair, null, null, null);
  }

  void prepare(IKeyPair keyPair, Amount fee, UInt32 Sequence,
      UInt32 lastLedgerSequence) {
    Blob pubKey = new Blob(keyPair.canonicalPubBytes());

    // This won't always be specified
//    if (lastLedgerSequence != null) {
//      txn.put(UInt32.LastLedgerSequence, lastLedgerSequence);
//    }
//    if (Sequence != null) {
//      txn.put(UInt32.Sequence, Sequence);
//    }
//    if (fee != null) {
//      txn.put(Amount.Fee, fee);
//    }

    txn.signingPubKeyFromBlob(pubKey);

    if (Transaction.CANONICAL_FLAG_DEPLOYED) {
      txn.setCanonicalSignatureFlag();
    }

    txn.checkFormat();
    signingData = txn.signingData();

    var hexStr = Util.formatBytesAsHexString(signingData);
    //print("签名前数据    : $hexStr");

    Function eq = const ListEquality().equals;
    if (previousSigningData != null && eq(signingData, previousSigningData)) {
      return;
    }
    try {
      var tmpSign = keyPair.signMessage(signingData);

      txn.txnSignature(new Blob(keyPair.signMessage(signingData)));

      BytesList blob = new BytesList();

      //  HalfSha512 id = HalfSha512.prefixed256(HashPrefix.transactionID);

      //    SHA512halfDigest digest = new SHA512halfDigest();

      var sha512 = new SHA512Digest();
      sha512.update(HashPrefix.transactionID.bytes(), 0,
          HashPrefix.transactionID.bytes().length);

      List<BytesSink> listBytesSink = new List<BytesSink>();

      listBytesSink.add(blob);

      txn.toBytesSink(new MultiSink(listBytesSink));

      tx_blob = blob.bytesHex();

      // print("tx_blob : $tx_blob");

      hash = Hash256.prefixedHalfSha512(HashPrefix.transactionID, blob.bytes());

//      var hexHash1 = hash.toString();
//      var hexHash = Util.formatBytesAsHexString(hash.toBytes());

      // print("hash : $hexHash1");

    } on Exception catch (e) {
      previousSigningData = null;
      throw new Exception(e);
    }

    previousSigningData = signingData;
  }

  TransactionType transactionType() {
    return txn.transactionType();
  }
}
